Type: Package
Package: INLAvaan
Title: Approximate Bayesian Latent Variable Analysis
Version: 0.2.5
Authors@R: c(
    person("Haziq", "Jamil", , "haziq.jamil@gmail.com", role = c("aut", "cre", "cph"),
           comment = c(ORCID = "0000-0003-3298-1010")),
    person("Håvard", "Rue", role = "ctb",
           comment = c(ORCID = "0000-0002-0222-1881", "Statistical and computational methodology")),
    person("Alvin", "Bong", role = "ctb",
           comment = "Initial site build")
  )
Description: Implements approximate Bayesian inference for Structural
    Equation Models (SEM) using a custom adaptation of the Integrated
    Nested Laplace Approximation (Rue et al., 2009)
    <doi:10.1111/j.1467-9868.2008.00700.x> as described in Jamil and Rue
    (2026a) <doi:10.48550/arXiv.2603.25690>. Provides a computationally
    efficient alternative to Markov Chain Monte Carlo (MCMC) for Bayesian
    estimation, allowing users to fit latent variable models using the
    'lavaan' syntax. See also the companion paper on implementation and
    workflows, Jamil and Rue (2026b) <doi:10.48550/arXiv.2604.00671>.
License: GPL (>= 3)
URL: https://inlavaan.haziqj.ml/, https://github.com/haziqj/INLAvaan
BugReports: https://github.com/haziqj/INLAvaan/issues
Depends: R (>= 3.5)
Imports: cli, graphics, lavaan (>= 0.6-19), methods, parallel, stats,
        utils
Suggests: blavaan, ggplot2, knitr, lme4, numDeriv, qrng, quarto, sn,
        testthat (>= 3.0.0), ucminf
VignetteBuilder: quarto
Config/Needs/website: rmarkdown
Config/testthat/edition: 3
Encoding: UTF-8
RoxygenNote: 7.3.3
NeedsCompilation: yes
Packaged: 2026-06-11 08:47:58 UTC; haziqj
Author: Haziq Jamil [aut, cre, cph] (ORCID:
    <https://orcid.org/0000-0003-3298-1010>),
  Håvard Rue [ctb] (ORCID: <https://orcid.org/0000-0002-0222-1881>,
    Statistical and computational methodology),
  Alvin Bong [ctb] (Initial site build)
Maintainer: Haziq Jamil <haziq.jamil@gmail.com>
Repository: CRAN
Date/Publication: 2026-06-11 09:10:02 UTC
Built: R 4.5.2; ; 2026-06-11 10:50:05 UTC; unix
