% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{is_same_function}
\alias{is_same_function}
\title{Helper function to check if two functions are the same}
\usage{
is_same_function(f, g)
}
\arguments{
\item{f, g}{Functions to compare.}
}
\value{
Logical.
}
\description{
Helper function to check if two functions are the same
}
\examples{
f1 <- function(x) { x^2 + 1 }
f2 <- function(x) { x^2 + 1 }
is_same_function(f1, f2)  # TRUE
}
