% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/standardisedsolution.R
\name{standardisedsolution}
\alias{standardisedsolution}
\alias{standardisedSolution}
\alias{standardizedsolution}
\alias{standardizedSolution}
\title{Standardised solution of a latent variable model}
\usage{
standardisedsolution(
  object,
  type = "std.all",
  se = TRUE,
  ci = TRUE,
  level = 0.95,
  postmedian = FALSE,
  postmode = FALSE,
  cov.std = TRUE,
  remove.eq = TRUE,
  remove.ineq = TRUE,
  remove.def = FALSE,
  nsamp = 250,
  ...
)

standardisedSolution(
  object,
  type = "std.all",
  se = TRUE,
  ci = TRUE,
  level = 0.95,
  postmedian = FALSE,
  postmode = FALSE,
  cov.std = TRUE,
  remove.eq = TRUE,
  remove.ineq = TRUE,
  remove.def = FALSE,
  nsamp = 250,
  ...
)

standardizedsolution(
  object,
  type = "std.all",
  se = TRUE,
  ci = TRUE,
  level = 0.95,
  postmedian = FALSE,
  postmode = FALSE,
  cov.std = TRUE,
  remove.eq = TRUE,
  remove.ineq = TRUE,
  remove.def = FALSE,
  nsamp = 250,
  ...
)

standardizedSolution(
  object,
  type = "std.all",
  se = TRUE,
  ci = TRUE,
  level = 0.95,
  postmedian = FALSE,
  postmode = FALSE,
  cov.std = TRUE,
  remove.eq = TRUE,
  remove.ineq = TRUE,
  remove.def = FALSE,
  nsamp = 250,
  ...
)
}
\arguments{
\item{object}{An object of class \link{INLAvaan}.}

\item{type}{If \code{"std.lv"}, the standardized estimates are based
on the variances of the (continuous) latent variables only.
If \code{"std.all"}, the standardized estimates are based
on the variances of both (continuous) observed and latent variables.
If \code{"std.nox"}, the standardized estimates are based
on the variances of both (continuous) observed and latent variables,
but not the variances of exogenous covariates.
Alternatively, \code{type} may be a vector of (observed) variable names
(for example \code{type = c("x1", "x2")}); in that case only the parameters
involving these variables are standardized (the other observed variables are
left unstandardized). This is a generalization of \code{"std.nox"}, where the
(observed) exogenous \code{x} variables are the ones left unstandardized.}

\item{se}{Logical. If TRUE, standard errors for the standardized parameters
will be computed, together with a z-statistic and a p-value.}

\item{ci}{If \code{TRUE}, confidence intervals are added to
the output.}

\item{level}{The confidence level required.}

\item{postmedian}{Logical; if TRUE, include posterior median in estimates.}

\item{postmode}{Logical; if TRUE, include posterior mode in estimates.}

\item{cov.std}{Logical. If \code{TRUE}, the (residual) observed covariances
are scaled by the square root of the \code{Theta} diagonal elements, and the
(residual) latent covariances are scaled by the square root of the
\code{Psi} diagonal elements. If \code{FALSE}, the (residual) observed
covariances are scaled by the square root of the diagonal elements of
the model-implied observed covariance matrix, and the (residual)
latent covariances are scaled similarly using the model-implied
covariance matrix of the latent variables. Documented explicitly here
(rather than inherited) because lavaan >= 0.7-1 renamed this and the
next three arguments to snake_case.}

\item{remove.eq}{Logical. If \code{TRUE}, filter the output by removing all
rows containing equality constraints, if any.}

\item{remove.ineq}{Logical. If \code{TRUE}, filter the output by removing all
rows containing inequality constraints, if any.}

\item{remove.def}{Logical. If \code{TRUE}, filter the output by removing all
rows containing parameter definitions, if any.}

\item{nsamp}{The number of samples to draw from the approximate posterior
distribution for the calculation of standardised estimates.}

\item{...}{Additional arguments sent to \code{lavaan::standardizedSolution()}.}
}
\value{
A \code{data.frame} containing standardised model parameters.
}
\description{
Standardised solution of a latent variable model
}
\examples{
HS.model <- "
  visual  =~ x1 + x2 + x3
  textual =~ x4 + x5 + x6
"
utils::data("HolzingerSwineford1939", package = "lavaan")

# Fit a CFA model with standardised latent variables
fit <- acfa(
  HS.model,
  data = HolzingerSwineford1939,
  test = "none",
  nsamp = 10,
  vb_correction = FALSE,
  verbose = FALSE
)
standardisedsolution(fit, nsamp = 10, se = FALSE, ci = FALSE)
}
\seealso{
\code{\link[=summary]{summary()}}, \code{\link[=coef]{coef()}}, \code{\link[=vcov]{vcov()}}
}
