% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/INLAvaan-class.R, R/method-others.R,
%   R/method-show.R, R/method-summary.R
\docType{class}
\name{INLAvaan-class}
\alias{INLAvaan-class}
\alias{coef}
\alias{coef,INLAvaan-method}
\alias{nobs}
\alias{nobs,INLAvaan-method}
\alias{show}
\alias{show,INLAvaan-method}
\alias{summary}
\alias{summary,INLAvaan-method}
\title{Class For Representing a (Fitted) Latent Variable Model}
\usage{
\S4method{coef}{INLAvaan}(object, ...)

\S4method{nobs}{INLAvaan}(object, ...)

\S4method{show}{INLAvaan}(object)

\S4method{summary}{INLAvaan}(
  object,
  header = TRUE,
  fit.measures = TRUE,
  estimates = TRUE,
  ci = TRUE,
  standardized = FALSE,
  standardised = standardized,
  rsquare = FALSE,
  postmedian = FALSE,
  postmode = FALSE,
  nmad = TRUE,
  kld = FALSE,
  vb_shift = FALSE,
  priors = TRUE,
  nd = 3L,
  ...
)
}
\arguments{
\item{object}{An object of class \code{INLAvaan}.}

\item{...}{Additional arguments passed to methods.}

\item{header}{Logical; if TRUE, print model fit information header.}

\item{fit.measures}{Logical; if TRUE, print fit measures (DIC and PPP).}

\item{estimates}{Logical; if TRUE, print parameter estimates table.}

\item{ci}{Logical; if TRUE (default), include 95\% credible intervals
(2.5\% and 97.5\% posterior quantiles) in the estimates table.}

\item{standardized}{Logical; if TRUE, include standardized estimates.}

\item{standardised}{Alias of \code{standardized}; either spelling is accepted
(the alias wins if both are supplied).}

\item{rsquare}{Logical; if TRUE, include R-square values for the observed
dependent variables, computed at the posterior mean point estimates.}

\item{postmedian}{Logical; if TRUE, include posterior median in estimates.}

\item{postmode}{Logical; if TRUE, include posterior mode in estimates.}

\item{nmad}{Logical; if TRUE (default), include the NMAD column for
skew-normal marginal fit quality.}

\item{kld}{Logical; if FALSE (default), omit the per-parameter KLD column.
Set to TRUE to show it.}

\item{vb_shift}{Logical; if FALSE (default), omit the VB shift column
(shift in units of posterior SD). Set to TRUE to show it.}

\item{priors}{Logical; if TRUE, include prior information in estimates.}

\item{nd}{Integer; number of decimal places to print for numeric values.}
}
\description{
This is a class that extends the \link[lavaan:lavaan-class]{lavaan} class. Several S4 methods are
available.
}
\section{Slots}{

\describe{
\item{\code{external}}{A list containing an \code{inlavaan_internal} object.}
}}

\examples{
\donttest{
HS.model <- "
  visual  =~ x1 + x2 + x3
  textual =~ x4 + x5 + x6
  speed   =~ x7 + x8 + x9
"
utils::data("HolzingerSwineford1939", package = "lavaan")
fit <- acfa(HS.model, HolzingerSwineford1939, std.lv = TRUE, nsamp = 100,
            test = "none", verbose = FALSE)

# Print basic info
fit

# Detailed summary
summary(fit)

# Extract coefficients
coef(fit)
}

}
\seealso{
\link[lavaan:lavaan-class]{lavaan}, \code{\link[=inlavaan]{inlavaan()}}, \code{\link[=acfa]{acfa()}}, \code{\link[=asem]{asem()}}, \code{\link[=agrowth]{agrowth()}}
}
