% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{as_fun_string}
\alias{as_fun_string}
\title{Convert function to single string}
\usage{
as_fun_string(f)
}
\arguments{
\item{f}{Function to convert.}
}
\value{
A single character vector representing the function.
}
\description{
Convert function to single string
}
\examples{
f <- function(x) { x^2 + 1 }
as_fun_string(f)
}
